//-----------------------------------------------------------------------------
// 
// Crypt :: Encryption routines
//
// Math routines for encrypting and decripting data
//
// Notes
//  (1) CryptTea    De/En-cryption method using the TEA algorithm
//
//-----------------------------------------------------------------------------

#include "Crypt.h"
#include <string.h>
#include <stdlib.h>

#define DBG_CTEA 0
#if (DBG_CTEA > 0)
#include <stdio.h>
#endif


CryptTea::CryptTea( const xstring pubkey[16], const xstring privkey[16] )
{ // create a new encryption routine, based on give public 
  // and private keys

    memcpy(publicKey,pubkey,16);
    memcpy(secretKey,privkey,16);
    s_buffer = 0;
    b_buffer = 0;
    sbuf_len = bbuf_len = 0;
}


CryptTea::CryptTea( const xstring pubkey[16] )
{ // create a new encryption routine, based on give public 
  // key. private key is only know to the routine.

    memcpy(publicKey,pubkey,16);
    memcpy(secretKey,"20010128AlexKoen",16);
    s_buffer = 0;
    b_buffer = 0;
    sbuf_len = bbuf_len = 0;
}


CryptTea::~CryptTea()
{
    if (s_buffer)
        free(s_buffer);

    if (b_buffer)
        free(b_buffer);
}


// Encryption -------------------------------------------


bool CryptTea::Encrypt( const xuint8 * cdata, xuint8 * xdata, xint32 datalen )
{

    xint32   i,j,p;
    xuint32  data[2];  // block buffer

  // encryption is done in blocks of 8 bytes. If the data in cdata is not
  // a multiple of 8 then it cannot be performed as encrypted data cannot be
  // truncated!

    if ( datalen % 8 )
        return false;

    j = 0;              // out-buffer pointer

    for ( i=0 ; i<datalen ; i+=8 )
    {
        data[0] = 0;
        data[1] = 0;

      // direct copy into the block buffer...

        memcpy( (void*)data, (void*)(cdata+i), 8 );

      // encode the password against the public key

        encodeBlock( (xuint32*)data, (xuint32*)publicKey );

      // encode the password against the secret key

        encodeBlock( (xuint32*)data, (xuint32*)secretKey );

      // place into encBuffer in this manner : data[0]{MSB ... LSB} data[1]{MSB ... LSB}
      // starting with data[0]MSB placed into encBuffer[0]

        for ( p=0 ; p<8 ; p++ )
        {
            xdata[j++] = (xuint8)( data[p/4] >> ( (3-(p%4)) * 8) );
        }
    }

    return true;
}


xuint8 * CryptTea::EncryptString( const xstring * cdata, xint32 & datalen )
{
    int   reqStrlen = strlen(cdata)+10;
    int   i,p;

    if (sbuf_len < reqStrlen )
    {
        s_buffer = (xstring *)realloc( s_buffer, reqStrlen );
        if (s_buffer)
        {
            sbuf_len = reqStrlen;
        }
    }

    if (!s_buffer)
    {
        datalen = 0;
        return NULL;
    }

    strcpy( s_buffer, cdata );
    
    reqStrlen  = strlen(cdata);
    p          = reqStrlen + 8 - (reqStrlen%8);

    for ( i = reqStrlen ; i < p ; i++ )
        s_buffer[i] = ' ';
    
    reqStrlen   = i;
    s_buffer[i] = '\0';        // for debugging ...

    #if (DBG_CTEA>=1)
    printf("    [CryptTea::EncryptString] padded to %d characters [%s]\n", reqStrlen, s_buffer );
    #endif

  // create the binary buffer

    if ( bbuf_len < reqStrlen)
    {
        b_buffer = (xuint8 *)realloc( b_buffer, reqStrlen );
        if ( b_buffer )
        {
            bbuf_len = reqStrlen;
        }
    }

    if (!b_buffer)
    {
        datalen = 0;
        return NULL;
    }

  // break up into 8-byte chunks and encrypt each chunk seperately

    for ( i=0 ; i<reqStrlen ; i+=8 )
    {
        if ( !Encrypt( (const xuint8 *)(s_buffer+i), b_buffer+i, 8 ) )
        {
            datalen = 0;
            return NULL;
        }
    }

    datalen = reqStrlen;
    return b_buffer;
}


bool CryptTea::Decrypt( const xuint8 * xdata, xuint8 * cdata, xint32 datalen )
{
    xint32   i,j,p;
    xuint32  data[2];  // block buffer

  // decryption is done in blocks of 8 bytes. If the data in cdata is not
  // a multiple of 8 then it cannot be performed as data is not valid.

    if ( datalen % 8 )
        return false;

    j = 0;

    for ( i=0 ; i<datalen ; i+=8 )
    {
        data[0] = 0;
        data[1] = 0;

      // big endian!

        for ( p=0 ; p<8 ; p++ )
        {
            data[p/4] |= ( (xuint32)(xdata[i+p]) << ( (3-(p%4)) * 8));
        }

      // decode the password against the secret key

        decodeBlock( (xuint32*)data, (xuint32*)secretKey);

      // decode the password against the public key

        decodeBlock( (xuint32*)data, (xuint32*)publicKey);

      // copy decrypted data out into cdata

        memcpy( (void*)(cdata+j), (void*)data, 8 );
        j += 8;
    }

    return true;
}


bool CryptTea::StripString( xstring * sdata, xint32 sdatalen )
{
    for ( int i = sdatalen-1 ; i>=0 ; i-- )
    {
        if ( sdata[i] == '\0' || sdata[i] == ' ' )
        {
            sdata[i] = '\0';
        }
        else 
        {
            break;
        }
    }
    return true;
}


xstring * CryptTea::GenerateAsciiString( const xuint8 * bdata, xint32 bdatalen )
{
    xint32 slen = (bdatalen+1) * 2;
    
    if (sbuf_len < slen )
    {
        s_buffer = (xstring *)realloc( s_buffer, slen );
        if (s_buffer)
        {
            sbuf_len = slen;
        }
    }

    if (!s_buffer)
    {
        return NULL;
    }

    memset( s_buffer, 0, sbuf_len );

    slen = 0;
    for ( int i = 0 ; i<bdatalen ; i++ )
    {
        bin2Ascii( bdata[i], s_buffer+slen );
        slen += 2;
    }

    return s_buffer;
}


xuint8 * CryptTea::DecodeAsciiString( const xstring * sdata, xint32 & bdatalen )
{
    xint32 slen = strlen(sdata);

    if ( slen % 1 )
        return NULL;  // must be multiple of 2

    bdatalen = slen>>1;

    if ( bbuf_len < bdatalen)
    {
        b_buffer = (xuint8 *)realloc( b_buffer, bdatalen );
        if ( b_buffer )
        {
            bbuf_len = bdatalen;
        }
    }

    if (!b_buffer)
    {
        bdatalen = 0;
        return NULL;
    }

    slen = 0;

    for ( int i = 0 ; i < bdatalen ; i++ )
    {
        b_buffer[i] = ascii2Bin( (const xstring *)(sdata+slen) );
        slen += 2;
    }

    return b_buffer;
}


// Low-level routines -----------------------------------


// encrypt a 64-bit data block (v) with a 128-bit key(k). result
// is stored back into v.
void CryptTea::encodeBlock( xuint32 * v, xuint32 * k )
{ // v[2], k[4] !
    register xuint32 y = v[0];
    register xuint32 z = v[1];
    register xuint32 sum = 0;
    register xuint32 delta = 0x9e3779b9;
    register xuint32 n = 32;

    while(n-->0)
    {
        y   += (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3];
        sum += delta;
        z   += (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3];
    }

    #if (DBG_CTEA>=2)
    char key[17];
    memset( key, 0, sizeof(key) );
    strncpy( key, (const char *)k, 16 );
    printf("    [CryptTea::encodeBlock] key[%s] {%08x%08x} -> {%08x%08x}\n", key, v[0], v[1], y, z ); 
    #endif

    v[0]=y;
    v[1]=z; // v' = E(v,k), v = data, k = key
}


// decrypt a 64-bit data block (v) with a 128-bit key(k). result
// is stored back into v.
void CryptTea::decodeBlock( xuint32 * v, xuint32 * k )
{ // v[2], k[4] !
    register xuint32 n = 32;
    register xuint32 sum;
    register xuint32 y = v[0];
    register xuint32 z = v[1];
    register xuint32 delta = 0x9e3779b9;

    sum = delta << 5 ;

    while(n-->0)
    {
        z   -= (y << 4 ^ y >> 5) + y ^ sum + k[sum >> 11 & 3];
        sum -= delta;
        y   -= (z << 4 ^ z >> 5) + z ^ sum + k[sum & 3];
    }

    v[0]=y;
    v[1]=z; // v' = D(v,k), v = data, k = key
}


xuint8 CryptTea::ascii2Bin( const xstring * an )
{ // convert a 2-byte ASCII string-val into its binary equivalent
  // eg. 00 -> 0, 1A -> 0x1a (26)

    xuint8  temp;
    xint8   alo,ahi;

  // convert string to uppercase

    alo = an[1];
    ahi = an[0];

    if ( alo >= 'a' && alo <= 'f' )  alo -= 32;
    if ( ahi >= 'a' && ahi <= 'f' )  ahi -= 32;

  // high nibble

    temp = ( ahi >= '0' && ahi <= '9') ? ahi - '0' : (ahi - 'A') + 10;
    temp <<= 4;

  // low nibble

    temp |= ( alo >= '0' && alo <= '9') ? alo - '0' : (alo - 'A') + 10;

  // result

    return temp;
}


const xint8 b2aTable[16] = 
{ 
  '0','1','2','3','4','5','6','7',
  '8','9','A','B','C','D','E','F'
};


void CryptTea::bin2Ascii( xuint8 b, xstring asc[2] )
{
    asc[0] = b2aTable[b / 16]; // high-nibble
    asc[1] = b2aTable[b % 16]; // low-nibble
}
