//-----------------------------------------------------------------------------
// 
// Crypt :: Encryption routines
//
// Math routines for encrypting and decripting data
//
// Notes
//  (1) CryptTea    De/En-cryption method using the TEA algorithm
//
//-----------------------------------------------------------------------------

#ifndef CRYPT_H
#define CRYPT_H


// Encryption engine types --------------------------------
typedef unsigned char   xuint8;
typedef char            xint8;
typedef unsigned int    xuint32;
typedef int             xint32;
typedef char            xstring;


class CryptTea
{
  public:

                CryptTea( const xstring pubkey[16], const xstring privkey[16] );
                // create a new encryption routine, based on give public 
                // and private keys

                CryptTea( const xstring pubkey[16] );
                // same, except private key is truly private!

               ~CryptTea();

  // Binary Encryption ------------------------------------

    bool        Encrypt( const xuint8 * cdata, xuint8 * xdata, xint32 datalen );
                // encrypt a block of data (cdata) into the xdata binary
                // buffer with a guaranteed length of datalen for both buffers.
                // note: encryption is performed 1:1 with no expansion of
                // length in the resultant data buffer.
                // IMPORTANT: datalen must be a multiple of 8 bytes.

    xuint8 *    EncryptString( const xstring * cdata, xint32 & datalen );
                // Similar to Encrypt() with the exception that it will
                // accept a NULL-terminated string of any length. The string
                // will be padded out to a multiple of 8 characters using
                // whitespace ("hello" ->"hello   "). The binary encrypted
                // result is placed into a local buffer and its pointer
                // is returned. datalen is set to the length of encrypted
                // binary data.
                // IMPORTANT : returning data is _not_ ACSII-encoded. Use the
                // GenerateAsciiString() method to perform the computation.
                // NOTE: If the cdata string is already a muliple of 8 characters
                // then an additional 8 whitespaces will be added.
                // (eg. "hellobof" -> "hellobof        ")

  // Binary Decryption ------------------------------------

    bool        Decrypt( const xuint8 * xdata, xuint8 * cdata, xint32 datalen );
                // decrypt data in the binary buffer xdata into the resultant 
                // binary buffer cdata. data is no expanded upon decryption
                // so both buffers must be of identical length 'datalen'.
                // IMPORTANT: datalen must be a multiple of 8 bytes.

    bool        StripString( xstring * sdata, xint32 sdatalen );
                // Decrypted strings (from Decrypt() ) may be padded with
                // whitespace characters. This is a simple method that will 
                // remove them for you. Declare the length of the string
                // in 'sdatalen'.

  // ASCII-Binary string format support -------------------

    xstring *   GenerateAsciiString( const xuint8 * bdata, xint32 bdatalen );
                // {binary} -> "string"
                // returns \0-terminated string representing the binary
                // number sequence in bdata. order is maintained such that
                // bdata[0] is the first 2 characters in the string (indices 0,1)
                // eg. array {00,a1,34,5d,ef,01} -> "00A1345DEF01\0"

    xuint8 *    DecodeAsciiString( const xstring * sdata, xint32 & bdatalen );
                // "string" -> {binary}
                // accepts a \0-terminated string and returns a pointer 
                // to an array holding the binary result. Length of the binary
                // array is set in bdatalen. Returns NULL if the given string does
                // not have an even (multiple of 2) length.
                // eg. string "00A1345DEF01\0" -> {00,a1,34,5d,ef,01} 

  private:

    void        encodeBlock( xuint32 * v, xuint32 * k );
                // encode 8 bytes of data. v & k are arrays of 2 (v[2],k[2])

    void        decodeBlock( xuint32 * v, xuint32 * k );
                // decode 8 bytesw of data. v & k are arrays of 2 (v[2],k[2])

    xuint8      ascii2Bin( const xstring * an );
                // convert a 2-byte ASCII string-val into its binary 
                // equivalent, eg. 00 -> 0, 1A -> 0x1a (26)

    void        bin2Ascii( xuint8 b, xstring asc[2] );
                // 0 -> "00", 15 -> "0F", 255 -> "FF" etc.

    xstring     publicKey[16];
    xstring     secretKey[16];

    xstring *   s_buffer;
    xuint8  *   b_buffer;
    xint32      sbuf_len;
    xint32      bbuf_len;

};

#endif // CRYPT_H
