// Accelerometer Transform Class

#include "Euler.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>


Euler::Euler() 
    : T_phi(3,3)
    , T_tta(3,3)
    , T_psi(3,3)
    , vp(3,1)
    , vpp(3,1)
{
  // Calibration Data
    
    calibrated = false;
    Phi = 0;
    Tta = 0;
    Psi = 0;
}


Euler::~Euler()
{
}


bool Euler::SetCal( float phi, float theta, float psi, bool flipP12 )
{ // update calibration angles. sets validation 
  // flag to enable routines requiring calibration.
  //   phi         angle of rotation about Z from the plane of v" (N) to 
  //               the +X axis, produces A' (Ap) (final)
  //   theta       angle of rotation about X from v' to -Z axis, 
  //               produces v" (vpp)
  //   psi         angle of rotation about Y from the 1G Gravitational 
  //               Vector (v) to the -Z axis, produces v' (vp)
  //   flipP12     FLAG, determine the order in which the first two 
  //               (static phase) angles are transformed.

    Phi = phi;
    Tta = theta;
    Psi = psi;
    calibrated = true;
    Flip_P12 = flipP12;
    return calibrated;
}


bool Euler::SetTrans()
{
    if (!calibrated)
        return false;   // not calibrated...

  // T_phi, rotation about Z

    T_phi.val(0,0) = cos(Phi);
    T_phi.val(0,1) = sin(Phi);
    T_phi.val(0,2) = 0;
    T_phi.val(1,0) = sin(Phi) * -1;
    T_phi.val(1,1) = cos(Phi);
    T_phi.val(1,2) = 0;
    T_phi.val(2,0) = 0;
    T_phi.val(2,1) = 0;
    T_phi.val(2,2) = 1;
    
  // T_tta, rotation about X
  
    T_tta.val(0,0) = 1;            
    T_tta.val(0,1) = 0;            
    T_tta.val(0,2) = 0;            
    T_tta.val(1,0) = 0;            
    T_tta.val(1,1) = cos(Tta);     
    T_tta.val(1,2) = sin(Tta);     
    T_tta.val(2,0) = 0;            
    T_tta.val(2,1) = sin(Tta) * -1;
    T_tta.val(2,2) = cos(Tta);     

  // T_psi, rotation about Y

    T_psi.val(0,0) = cos(Psi);     
    T_psi.val(0,1) = 0;            
    T_psi.val(0,2) = sin(Psi);     
    T_psi.val(1,0) = 0;            
    T_psi.val(1,1) = 1;            
    T_psi.val(1,2) = 0;            
    T_psi.val(2,0) = sin(Psi) * -1;
    T_psi.val(2,1) = 0;            
    T_psi.val(2,2) = cos(Psi);     

    return true;
}


void Euler::ResetCal()
{ // call this to invalidate any calibration values. 
  // causes all routine calls requiring calibration 
  // to fail.

    calibrated = Flip_P12 = false;
    Phi = 0;
    Tta = 0;
    Psi = 0;
}


bool Euler::Transform( Matrix & Ap, Matrix & A )
{
    if (!calibrated)
        return false;   // not calibrated...

    if (Flip_P12)
    {
        Mult(vp,T_tta,A);       // vp  = T_tta * A
        Mult(vpp,T_psi,vp);     // vpp = T_psi * vp
    }
    else
    {
        Mult(vp,T_psi,A);       // vp  = T_psi * A
        Mult(vpp,T_tta,vp);     // vpp = T_tta * vp
    }

    Mult(Ap,T_phi,vpp);     // Ap  = T_phi * vpp

    return true;
}


bool Euler::Cal( float & phi, float & theta, float & psi, bool & flipP12 )
{ // read calibration angles. return valid status of values.

    phi     = Phi;
    theta   = Tta;
    psi     = Psi;
    flipP12 = Flip_P12;
    return calibrated;
}


bool Euler::IsCalibrated() { return calibrated; }
  // returns true if valid calibration values are 
  // loaded.
