//-----------------------------------------------------------------------------
//
// Euler :: Euler Transformation Class, for Accelerometer data analysis.
//
// Processing class for transformation of vectors from one Cartesian frame
// to another. Includes tools for determining the angular differences between
// frames (calibration)
//
// Notes
//  (1) Transform Convention Sub-type : y-x-z
//
//  (2) Parameter Definitions
//
//          Angle       Defines
//          ----------  ---------------------------------------
//          psi         angle of rotation about Y from the 1G Gravitational
//                      Vector (v) to the -Z axis, produces v'
//          theta       angle of rotation about X from v' to -Z axis,
//                      produces v"
//          phi         angle of rotation about Z from the plane of v" (N) to
//                      the +X axis, produces v"' (final)
//          Th_xz       angle in XZ plane of the position vector Pp
//          Th_yz       angle in YZ plane of the position vector Pp
//
//          Frame       Defines
//          ----------  ---------------------------------------
//          (xyz)       Coordinate Frame for the WT7000
//          (XYZ)       Coordinate Frame for the Vehicle where,
//                          +X points to front, +Z points vertically up
//
//          Vector      Defines
//          ----------  ---------------------------------------
//          A           A_x  accel. in x-axis
//                      A_y  accel. in y-axis
//                      A_z  accel. in z-axis
//          Ap          Ap_x accel. in X-axis
//                      Ap_y accel. in Y-axis
//                      Ap_z accel. in Z-axis
//          Pp          Pp_x pos. in X-axis
//                      Pp_y pos. in Y-axis
//                      Pp_z pos. in Z-axis
//          V           V_x  1G Gravity Vector in x-axis
//                      V_y  1G Gravity Vector in y-axis
//                      V_z  1G Gravity Vector in z-axis
//
//-----------------------------------------------------------------------------

#ifndef EULER_H
#define EULER_H

#ifdef __WIN32__
#endif

#ifdef PIKA
#endif

#include "MatrixMath.h"
#include "TrigMath.h"

// Accelerometer Transform Class

class Euler
{
  public:

                            Euler();
                           ~Euler();

  // Calibration Routines -----------------------

    bool                    SetCal( float phi, float theta, float psi, bool flipP12 );
                            // update calibration angles. sets validation
                            // flag to enable routines requiring calibration.
                            // *NEW* set 'flipP12' if theta is determined before psi,
                            // clear 'flipP12' if psi is the first angle to be
                            // measured.

    bool                    SetTrans();
                            // update transforms (T_phi,T_tta,T_psi) from the
                            // calibration angles supplied by SetCal().
                            // Note: Transforms can be adjusted individually
                            //       as they are public variables.

    void                    ResetCal();
                            // call this to invalidate any calibration values.
                            // causes all routine calls requiring calibration
                            // to fail.

  // Transform(s) -------------------------------

    bool                    Transform( Matrix & Ap, Matrix & A );
                            // Full Euler Transform; (Ap) = T * (A)

  // Calibration Checks -------------------------

    bool                    Cal( float & phi, float & theta, float & psi, bool & flipP12 );
                            // read calibration angles.

    bool                    IsCalibrated();
                            // returns true if valid calibration values are
                            // loaded.

  // Transform Matrices (public) ----------------

    Matrix                  T_phi;      // T1(3,3) : Transform for angle 'Phi'
    Matrix                  T_tta;      // T2(3,3) : Transform for angle 'Theta'
    Matrix                  T_psi;      // T3(3,3) : Transform for angle 'Psi'

  private:

  // Calibration Data ---------------------------

    bool                    calibrated; // set to true when Orientation angles are valid

    bool                    Flip_P12;
                            // when this is set, compute Tta -> Psi -> Phi. If clear
                            // then compute Psi -> Tta -> Phi.

    float                   Phi;        // Orientation Angles
    float                   Tta;
    float                   Psi;

  // Intermediate Data (3,1) --------------------

    Matrix                  vp;
    Matrix                  vpp;

};

#endif // EULER_H
