//-----------------------------------------------------------------------------
// 
// Matrix.h :: Simple 2D vector class
//
// A simple 2D (row,col) vector class, to support vector transformations.
//
// Notes
//  (1)     Operators (+, *) observe matrix alignment and order. Operator will
//          fail and return [0] if it cannot perform the operation.
//  (2)     Scalar multiplication _must_ have the first argument as the vector,
//          eg. x = (a * b) requires 'a' be the vector and 'b' be the scalar.
//  (3)     Operator overloading of '=' implemented. The object receiving the
//          result of the computation can be dimensionless, its dimension will
//          be established during execution of the equality operator. Note that
//          any existing dimensioned data in the receiving object will be lost
//          and its configuration changed.
//
//-----------------------------------------------------------------------------

#ifdef __WIN32__
#endif

#ifdef PIKA
#endif

#include "Matrix.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>



Matrix::Matrix()
{
    Reset();
}


Matrix::Matrix( int rowMax, int colMax )
{
    Reset();
    dim(rowMax,colMax);
}


Matrix::~Matrix()
{
    if ( mdata ) 
        delete mdata;
}


void Matrix::dim( int rowMax, int colMax )
{
    rowcount = rowMax;
    colcount = colMax;

    if (mdata)
        delete mdata;

    mdata = new float[rowMax*colMax];
    
    Clear();
}


void Matrix::dim( int rowMax, int colMax, float * data )
{
    rowcount = rowMax;
    colcount = colMax;

    if (mdata)
        delete mdata;

    mdata = new float[rowMax*colMax];
    memcpy( mdata, data, rowMax * colMax * sizeof(float) );
}


float & Matrix::val( int row, int col )
{
    if ( mdata && (row < rowcount) && (col < colcount) )
        return mdata[ (row * colcount) + col ];
    else
        return ferr;   // error
}


int Matrix::RowSz() { return rowcount; }


int Matrix::ColSz() { return colcount; }


void Matrix::Copy( Matrix & M )
{ // Copy data in supplied matrix into mine,
  // reformatting matrix size if required.

    if ( (rowcount != M.RowSz()) ||
         (colcount != M.ColSz()) )
    { // re-format to the correct size
        dim( M.RowSz(), M.ColSz() );
    }

    for ( int r = 0 ; r < rowcount ; r++ )
        for ( int c = 0 ; c < colcount ; c++ )
            val(r,c) = M.val(r,c);

}


float Matrix::Mag()
{
    return Mag(0);
}


float Matrix::Mag( int c )
{
    float vl = 0;
    float rc = 0;

    if ( c >= colcount )
        return ferr;

    for ( int i = 0 ; i < rowcount ; i++ )
    {
        rc = val(i,c);
        vl += (rc * rc);
    }

    return ( sqrt( vl ) );
}


float Matrix::Mag2D( int r1, int r2, int c )
{ // Planar Magnitude, based on axis r1 and r2 in col c.

    if ( (r1 >= rowcount) || (r2 >= rowcount) || (c >= colcount) )
        return ferr;

    float v1 = val(r1,c);
    float v2 = val(r2,c);

    return sqrtf( v1*v1 + v2*v2 );
}


float Matrix::Abs( int r, int c )
{ // Compute the magnitude of the element at (r,c)

    if ( (r >= rowcount) || (c >= colcount) )
        return ferr;

    return (val(r,c) >= 0) ? val(r,c) : (-1.0) * val(r,c);
}


void Matrix::Sum( Matrix & M )
{ // this = this + M
    
    if ( (rowcount == M.RowSz()) && (colcount == M.ColSz()) )
    { // perform r,c by r,c addition
        
        for ( int r = 0 ; r < rowcount ; r++ )
            for ( int c = 0 ; c < colcount ; c++ )
                val(r,c) += M.val(r,c);
    }
}


void Matrix::Div_m_s( float s )
{ // Divide : (Matrix / Scalar)
  // this = this / s
  // perform r,c by r,c division
        
    for ( int r = 0 ; r < rowcount ; r++ )
        for ( int c = 0 ; c < colcount ; c++ )
            val(r,c) = val(r,c) / s;
}


void Matrix::PeakDet_e( Matrix & M, COMP cmp )
{ // Perform element-by-element peak detection
  // eg. c == {greater-than (>)}
  // IF ( M.val(r,c) > this.val(r,c) ) THEN
  //     this.val(r,c) = M.val(r,c)
  // Note: inequality _is_not_ over the whole vector
  // Note: valid c is {LESS_THAN,GTR_THAN} only

    for ( int r = 0 ; r < rowcount ; r++ )
    {
        for ( int c = 0 ; c < colcount ; c++ )
        {
            if ( !cmpStarted || 
                 ( (cmp == LESS_THAN) && (M.val(r,c) < val(r,c)) ) ||  
                 ( (cmp == GTR_THAN)  && (M.val(r,c) > val(r,c)) ) )
            {
                val(r,c) = M.val(r,c);
            }
        }
    }
    cmpStarted = true;
    return;
}


bool Matrix::Ok() { return !error; }


bool Matrix::Error() { return error; }


void Matrix::Failed() { error = true; }


void Matrix::Clear()
{
    error      = false;
    cmpStarted = false;

    if (mdata && rowcount && colcount)
    {
        memset( mdata, 0, rowcount * colcount * sizeof(float) );
    }
}


void Matrix::Reset()
{
    rowcount   = 0;
    colcount   = 0;
    error      = false;
    ferr       = 0;
    mdata      = NULL;
    cmpStarted = false;
}

