//-----------------------------------------------------------------------------
// 
// Matrix.h :: Simple 2D vector class
//
// A simple 2D (row,col) vector class, to support vector transformations.
//
// Notes
//  (1)     Operators (+, *) observe matrix alignment and order. Operator will
//          fail and return [0] if it cannot perform the operation.
//  (2)     Scalar multiplication _must_ have the first argument as the vector,
//          eg. x = (a * b) requires 'a' be the vector and 'b' be the scalar.
//  (3)     Operator overloading of '=' implemented. The object receiving the
//          result of the computation can be dimensionless, its dimension will
//          be established during execution of the equality operator. Note that
//          any existing dimensioned memory in the receiving object will be lost 
//          if the new matrix configuration is different from the existing one.
//              eg.     if x has configuration (2,2) then,
//                      x = a(1,1)  : (2,2) matrix is deleted, replaced with (1,1)
//                      x = a(2,2)  : (2,2) matrix remains and is set to 'a'.
//
//-----------------------------------------------------------------------------

#ifndef MATRIX_H
#define MATRIX_H

#include <math.h>

#ifdef __WIN32__
#endif

#ifdef PIKA
#endif


// dimensional conventions, for simplifying calls into the matrix
#define ROW_X 0
#define ROW_Y 1
#define ROW_Z 2

// conventiional angles (3D)
    // ANGLE_<ref>_<plane>: angle from <ref> to vector in plane <plane> from column 0
#define ANGLE_X_XY AngleRad(ROW_X,ROW_Y,0)
#define ANGLE_X_XZ AngleRad(ROW_X,ROW_Z,0)
#define ANGLE_Y_YZ AngleRad(ROW_Y,ROW_Z,0)
#define ANGLE_Y_XY AngleRad(ROW_Y,ROW_X,0)
#define ANGLE_Z_XZ AngleRad(ROW_Z,ROW_X,0)
#define ANGLE_Z_YZ AngleRad(ROW_Z,ROW_Y,0)
    // ANGLE_<ref>_<plane>(col): angle from <ref> to vector in plane <plane> from column (col)
#define ANGLE_X_XYC(col) AngleRad(ROW_X,ROW_Y,col)
#define ANGLE_X_XZC(col) AngleRad(ROW_X,ROW_Z,col)
#define ANGLE_Y_YZC(col) AngleRad(ROW_Y,ROW_Y,col)


// Comparision Enumeration
enum COMP
{
    LESS_THAN = 0,          // <
    LESS_THAN_OR_EQUAL,     // <=
    IS_EQUAL,               // ==
    GTR_THAN_OR_EQUAL,      // >=
    GTR_THAN,               // >
    INVALID                 // ???
};

class Matrix
{
  public:

  // Standard Constructors / Destructors

                            Matrix();
                            Matrix( int rowMax, int colMax );
                           ~Matrix();

    void                    dim( int rowMax, int colMax );
    void                    dim( int rowMax, int colMax, float * data );

  // Data Access

    float &                 val( int row, int col );
    int                     RowSz();
    int                     ColSz();

    void                    Copy( Matrix & M );
                            // Copy data in supplied matrix into mine,
                            // reformatting matrix size if required.

  // Vector Analysis

    float                   Mag();          
                            // Vector Magnitude (n,0) (for all n, col = 0)

    float                   Mag( int c );
                            // Vector Magnitude (n,c) (for all n, for col c)

    float                   Mag2D( int r1, int r2, int c );
                            // Planar Magnitude, based on axis r1 and r2 in
                            // col c.

    float                   Abs( int r, int c );
                            // Compute the magnitude of the element at (r,c)

  // Vector Math

    void                    Sum( Matrix & M );
                            // this = this + M

    void                    Div_m_s( float s );
                            // Divide : (Matrix / Scalar)
                            // this = this / s

    void                    PeakDet_e( Matrix & M, COMP c );
                            // Perform element-by-element peak detection
                            // eg. c == {greater-than (>)}
                            // IF ( M.val(r,c) > this.val(r,c) ) THEN
                            //     this.val(r,c) = M.val(r,c)
                            // Note: inequality _is_not_ over the whole vector
                            // Note: valid c is {<,>} only

  // Status

    bool                    Ok();
                            // returns true, when valid

    bool                    Error();        
                            // returns true, when invalid

    void                    Failed();       
                            // call to invalidate this object.

  // Misc

    void                    Clear();
                            // zero all data & error flag in the matrix. does not clear row & col settings or delete memory

    void                    Reset();        
                            // zero all values, zero row & col, set matrix data pointer to NULL

  private:

    bool                    error;          // set true if matrix operation cannot be completed
    float                   ferr;           // set to zero
    int                     rowcount;
    int                     colcount;
    float *                 mdata;          // access as a 2D array, [row][col]
    bool                    cmpStarted;     // set to true when a compare is done the first time after a Clear() or Reset()
};

#endif // MATRIX_H
