//-----------------------------------------------------------------------------
// 
// TrigMath :: Trigonometric Math Routines
//
// Math routines for computing and converting angles.
//
// Notes
//  (1) Used as a base class for Euler (these functions used to be in Euler.h,
//      if Euler was required explicitly for access to these functions then
//      consider just incluing this header instead).
//
//-----------------------------------------------------------------------------

#include "TrigMath.h"
#include <math.h>


float QuadAngle( float v, float h )
{
    double dv  = v;
    double dh  = h;
    double res = 0;
    double dm  = sqrt( (dv * dv) + (dh * dh) );

    if ( dh >= 0 && dv >= 0 )
    { // Q-I
        res = acos( dv/dm );
    }
    else if ( dh >= 0 && dv < 0 )
    { // Q-IV
        res = PI - acos( -dv/dm );
    }
    else if ( dh < 0 && dv < 0 )
    { // Q-III
        res = PI + acos( -dv/dm );
    }
    else
    { // Q-II
        res = 2 * PI - acos( dv/dm );
    }

    return (float)res;
}


// An improvement over QuadAngle(), this fcn computes angle relative to 
// ref. axis (R). Orthonganal axis to (R) is labeled (O). Return angle 
// is ranged to +/- 180 relative to ref. axis.
float QuadAngleRA( float R, float O )
{
    double r  = R;
    double o  = O;
    double res = 0;
    double m  = sqrt( (r * r) + (o * o) );

    if ( r >= o )
    { // base the angle on the strongest axis, R
      // angle is in Q-I or Q-II
        res = acos( r/m );
      // invert for -ve angles (CW from R) -> Q-III, Q-IV
        if ( o < 0 )
            res = -res;
    }
    else
    { // base the angle on the strongest axis, O
      // angle is in Q-I or Q-VI
        res = asin( o/m );
      // inversion a bit more complicated, need to reflect into Q-II,Q-III
        if ( r < 0 )
        {
            double corr = PI;
            if ( o < 0 )
                corr = -corr;  // -PI
            res = corr - res;
        }
    }
    return (float)res;
}


bool Posn( Matrix & Ap, float & th_xz, float & th_yz )
{

#if (INVERT_Z_AXIS>0)
    float z = Ap.val(ROW_Z,0) * -1;   // invert, Gravity will be -1
#else
    float z = Ap.val(ROW_Z,0);
#endif

  // zx plane --------------

    th_xz = QuadAngle( z, Ap.val(ROW_X,0) );

    if ( th_xz > PI )
        th_xz = th_xz - (2 * PI);

  // zy plane --------------

    th_yz = QuadAngle( z, Ap.val(ROW_Y,0) );

    if ( th_yz > PI )
        th_yz = th_yz - (2 * PI);

    return true;
}


float RadToDeg( float rad )
{
    return ( rad * DEG_RAD ); 
}


float DegToRad( float deg )
{
    return ( deg * RAD_DEG ); 
}

// Taylor approx. up to x^3
float FastArcTan( float y, float x )
{
    bool neg_x = (x<0) ? true : false;
    bool neg_y = (y<0) ? true : false;

    if ( neg_x )
    {
        x = -x;
    }

    if ( neg_y )
    {
        y = -y;
    }

    if ( x == 0 )
    {
        return ( y>= 0) ? PI/2 : -PI/2; 
    }
    
    float a     = y/x;
    float a_cub = a * a * a;
    float res   = 0;
    
    if ( a >= 1 )
    {
        res  = (PI/2) - (1/a) + (0.33333333 / a_cub);
    }
    else
    {
        res = a - (a_cub * 0.33333333);
    }

    if ( neg_x )
    { // x < 0
        if ( neg_y )
        { // y < 0
            return (-PI/2) + res;
        }
        else
        { // y >= 0
            return (PI/2) - res;
        }
    }
    else
    { // x >= 0
        if ( neg_y )
        { // y < 0
            return -res;
        }
        else
        { // y >= 0
            return res;    
        }
    }
}


float tmMag( float a, float b )
{
    return (float)sqrt( (a * a) + (b * b) );
}


float tmMag( float a, float b, float c )
{
    return (float)sqrt( (a * a) + (b * b) + (c * c) );
}

