//-----------------------------------------------------------------------------
// 
// TrigMath :: Trigonometric Math Routines
//
// Math routines for computing and converting angles.
//
// Notes
//  (1) Used as a base class for Euler (these functions used to be in Euler.h,
//      if Euler was required explicitly for access to these functions then
//      consider just incluing this header instead).
//
//-----------------------------------------------------------------------------

#ifndef TRIGMATH_H
#define TRIGMATH_H

// Not using class methods, for simplicity.

#include "Matrix.h"

#ifndef PI
 #define PI  3.141592654
#endif

// (360 / 2PI )
#define DEG_RAD 57.295779513
// (2PI / 360)
#define RAD_DEG 0.017453293


// determine an angle from +h axis for the given vector broken down into a 
// vertical and horizontal component.
// ---------------------------------------------------------------------------
// IMPORTANT : angles do not conform to the standard (which is increasing 
// ccw), the quadangle starts at 0deg on the +v axis and increases CW, so +h 
// marks +90deg, -v marks +180deg, -h marks +270deg. There are no -ve angles 
// computed, range is (0 - 359.999)deg.
// ===========================================================================
float   QuadAngle( float v, float h );

// An improvement over QuadAngle(), this fcn computes angle relative to 
// ref. axis (R).
// ---------------------------------------------------------------------------
// Reference axis : R
// Orthognal axis : O
// Return Angle   : Relative to R, can be { 0 .. +/-PI } [Radians]
// NOTE: 1) determination of the strongest axis (the larger magnitude axis) is 
//          _not_ required.
//       2) R & O both represent positive axes in a Cartesian co-ordinate 
//          system.
//       3) +ve angle increases from +R to +O, negative is from +R to -O
// ===========================================================================
float QuadAngleRA( float R, float O );

// read orientation (angles) from Ap
// ---------------------------------------------------------------------------
// Used to read out 2 angles from the (3x1) matrix 'Ap'.
// Angle th_xz [Radians] is the angular difference, in the XZ plane, between
// the vector in Ap and the Z-axis (0,0,1). 
// Angle th_yz [Radians] is same as th_xz except that it is the angle in the 
// YZ plane.
// Range : both angles are range limited to {-PI ... +PI}
// ---------------------------------------------------------------------------
// INVERT_Z_AXIS :: 0 = Keep Z-axis polarity as-is.
//                  1 = Invert Z-axis
#define INVERT_Z_AXIS 0
// ===========================================================================
bool    Posn( Matrix & Ap, float & th_xz, float & th_yz );


// convert radian value to degrees
// ---------------------------------------------------------------------------
// Accepts a floating point value in Radians and returns angle in degrees.
// ===========================================================================
float   RadToDeg( float rad );


// convert degree value to radians
// ---------------------------------------------------------------------------
// Accepts a floating point value in Degrees and returns angle in radians.
// ===========================================================================
float   DegToRad( float deg );


// Calculate Tan-1(y/x) using a fast approximation
// ---------------------------------------------------------------------------
// Returns result in Radians.
// ===========================================================================
float   FastArcTan( float y, float x );

// Calculate a magnitude for 2 axes
// ---------------------------------------------------------------------------
// Returns magnitude.
// ===========================================================================
float   tmMag( float a, float b );

// Calculate a magnitude for 3 axes
// ---------------------------------------------------------------------------
// Returns magnitude.
// ===========================================================================
float   tmMag( float a, float b, float c );

#endif //TRIGMATH_H

